#!/usr/bin/env bash

# Note: The repetition of tile types in the tiles list weights repeated tile types to appear more frequently.

declare    tiles="gggggggggggggggggggggggggpppppppppppppppppppppfffffffffffffffffhhhhhhhsjdm" IFS=$'\n' zeroPad="0000"
declare -a terrainLayer e00Layer e01Layer e02Layer riverLayer resLayer
declare -i i j mapWidth=100 mapHeight=50 startPosCount=0



# Function(s)
#  1 in 10 chance of river on each land tile which is ~10% of the land tiles
RandomRiver() {
	if [ $(( ${RANDOM} % 10 )) -eq 0 ] ; then
		riverLayer[${i}]="${riverLayer[${i}]:0:${1}}2${riverLayer[${i}]:$((${1}+1))}"
	fi
}



usage="Usage:	$(basename ${0}) [-w width] [-h height]\n\
	width is width in tiles. Default value: 100\n\
	height is height in tiles. Default value: 50\n"

until [ -z "${1}" ] ; do
	case "${1}" in
		-w | -W | -x | -X | --width		)	shift ; mapWidth="${1}" ;;
		-h | -H | -y | -Y | --height		)	shift ; mapHeight="${1}" ;;
		--help 					)	printf "${usage}" ; exit 1 ;;
		* 					)	printf "${usage}" ; exit 1 ;;
	esac
	shift
done

if [ ${mapWidth} -lt 16 -o ${mapHeight} -lt 16 ] ; then
	echo "FreeCiv minimum map width/height is 16"
	exit 1
fi

# Create map layer arrays other than terrain layer
riverLayer[0]=$(temp=$(printf "%*s" ${mapWidth}) ; echo "${temp// /0}")
e00Layer[0]="${riverLayer[0]}"
e01Layer[0]="${riverLayer[0]}"
e02Layer[0]="${riverLayer[0]}"
resLayer[0]="$(printf "%*s" ${mapWidth})"

for ((i=1;i<${mapHeight};i++)) ; do
	e00Layer[${i}]="${riverLayer[0]}"
	e01Layer[${i}]="${riverLayer[0]}"
	e02Layer[${i}]="${riverLayer[0]}"
	riverLayer[${i}]="${riverLayer[0]}"
	resLayer[${i}]="${resLayer[0]}"
done


# Make waffle grid terrain map
for ((i=0;i<${mapHeight};i++)) ; do
	if [ ${i} -eq $(( (${i} / 2) * 2 )) ] ; then
		# Even lines
		for ((j=0;j<${mapWidth};j+=2)) ; do
			terrainLayer[${i}]="${terrainLayer[${i}]}${tiles:$((${RANDOM}%${#tiles})):1} "
			RandomRiver ${j}
		done
	else
		# Odd lines
		for ((j=0;j<${mapWidth};j+=2)) ; do
			terrainLayer[${i}]="${terrainLayer[${i}]} ${tiles:$((${RANDOM}%${#tiles})):1}"
			RandomRiver $((${j}+1))
		done
	fi
done


# Output the map for copy/paste to a FreeCiv ".sav" file

# t (terrain) layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "t${zeroPad:${#i}}${i}=\"${terrainLayer[${i}]}\""
done

# Start positions list
echo "startpos_count=${startPosCount}"
echo "startpos={\"x\",\"y\",\"exclude\",\"nations\""
echo -n "${startPosOutput}"
echo '}'

# e00_ Irrigation, mine, pollution, oil wells layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e00_${zeroPad:${#i}}${i}=\"${e00Layer[${i}]}\""
done

# e01_ Hut, farmland, fallout, fortress layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e01_${zeroPad:${#i}}${i}=\"${e01Layer[${i}]}\""
done

# e02_ Rail, airbase, buoy, ruins layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e02_${zeroPad:${#i}}${i}=\"${e02Layer[${i}]}\""
done

# e03_ River and road layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e03_${zeroPad:${#i}}${i}=\"${riverLayer[${i}]}\""
done

# res (resources) layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "res${zeroPad:${#i}}${i}=\"${resLayer[${i}]}\""
done


exit 0
